@@ -132,6 +132,15 @@ static VALUE cEngineInfo,
   cImportStatus,
   cImportResult;
 
+#if defined(GPGME_VERSION_NUMBER) && GPGME_VERSION_NUMBER >= 0x010500
+static VALUE
+rb_s_gpgme_get_dirinfo (VALUE dummy, VALUE vwhat)
+{
+  const char *result = gpgme_get_dirinfo (StringValueCStr(vwhat));
+  return result ? rb_str_new2 (result) : Qnil;
+}
+#endif
+
 static VALUE
 rb_s_gpgme_check_version (VALUE dummy, VALUE vreq)
 {
@@ -613,6 +622,54 @@ rb_s_gpgme_get_passphrase_cb (VALUE dummy, VALUE vctx, VALUE rpassfunc,
   return Qnil;
 }
 
+#if defined(GPGME_VERSION_NUMBER) && GPGME_VERSION_NUMBER >= 0x010600
+static gpgme_error_t
+status_cb (void *hook, const char *keyword, const char *args)
+{
+  VALUE vcb = (VALUE)hook, vstatusfunc, vhook_value;
+
+  vstatusfunc = RARRAY_PTR(vcb)[0];
+  vhook_value = RARRAY_PTR(vcb)[1];
+
+  rb_funcall (vstatusfunc, rb_intern ("call"), 3,
+	      vhook_value,
+	      keyword ? rb_str_new2 (keyword) : Qnil,
+	      args ? rb_str_new2 (args) : Qnil);
+  return gpgme_err_make (GPG_ERR_SOURCE_USER_1, GPG_ERR_NO_ERROR);
+}
+
+static VALUE
+rb_s_gpgme_set_status_cb (VALUE dummy, VALUE vctx, VALUE vstatusfunc,
+			  VALUE vhook_value)
+{
+  gpgme_ctx_t ctx;
+  VALUE vcb = rb_ary_new ();
+
+  rb_ary_push (vcb, vstatusfunc);
+  rb_ary_push (vcb, vhook_value);
+  /* Keep a reference to avoid GC. */
+  rb_iv_set (vctx, "@status_cb", vcb);
+
+  UNWRAP_GPGME_CTX(vctx, ctx);
+  if (!ctx)
+    rb_raise (rb_eArgError, "released ctx");
+  gpgme_set_status_cb (ctx, status_cb, (void*)vcb);
+  return Qnil;
+}
+
+static VALUE
+rb_s_gpgme_get_status_cb (VALUE dummy, VALUE vctx, VALUE rstatusfunc,
+			  VALUE rhook_value)
+{
+  VALUE vcb = rb_iv_get (vctx, "@status_cb");
+
+  /* No need to call gpgme_get_status_cb. */
+  rb_ary_store (rstatusfunc, 0, RARRAY_PTR(vcb)[0]);
+  rb_ary_store (rhook_value, 0, RARRAY_PTR(vcb)[1]);
+  return Qnil;
+}
+#endif
+
 static void
 progress_cb (void *hook, const char *what, int type, int current, int total)
 {
@@ -700,6 +757,35 @@ rb_s_gpgme_get_pinentry_mode (VALUE dummy, VALUE vctx)
 }
 #endif
 
+#if defined(GPGME_VERSION_NUMBER) && GPGME_VERSION_NUMBER >= 0x010600
+static VALUE
+rb_s_gpgme_set_offline (VALUE dummy, VALUE vctx, VALUE vyes)
+{
+  gpgme_ctx_t ctx;
+
+  UNWRAP_GPGME_CTX(vctx, ctx);
+  if (!ctx)
+    rb_raise (rb_eArgError, "released ctx");
+
+  gpgme_set_offline (ctx, vyes == Qtrue);
+  return Qnil;
+}
+
+static VALUE
+rb_s_gpgme_get_offline (VALUE dummy, VALUE vctx)
+{
+  gpgme_ctx_t ctx;
+  int yes;
+
+  UNWRAP_GPGME_CTX(vctx, ctx);
+  if (!ctx)
+    rb_raise (rb_eArgError, "released ctx");
+
+  yes = gpgme_get_offline (ctx);
+  return yes ? Qtrue : Qfalse;
+}
+#endif
+
 static VALUE
 rb_s_gpgme_op_keylist_start (VALUE dummy, VALUE vctx, VALUE vpattern,
 			     VALUE vsecret_only)
@@ -809,6 +895,10 @@ save_gpgme_key_attrs (VALUE vkey, gpgme_key_t key)
         rb_iv_set (vsubkey, "@fpr", rb_str_new2 (subkey->fpr));
       rb_iv_set (vsubkey, "@timestamp", LONG2NUM(subkey->timestamp));
       rb_iv_set (vsubkey, "@expires", LONG2NUM(subkey->expires));
+#if defined(GPGME_VERSION_NUMBER) && GPGME_VERSION_NUMBER >= 0x010500
+      if (subkey->curve)
+	rb_iv_set (vsubkey, "@curve", rb_str_new2 (subkey->curve));
+#endif
       rb_ary_push (vsubkeys, vsubkey);
     }
   vuids = rb_ary_new ();
@@ -1192,6 +1282,9 @@ rb_s_gpgme_op_import_result (VALUE dummy, VALUE vctx)
     rb_raise (rb_eArgError, "released ctx");
 
   result = gpgme_op_import_result (ctx);
+  if (!result)
+    rb_raise (rb_eArgError, "no result");
+
   vresult = rb_class_new_instance (0, NULL, cImportResult);
   rb_iv_set (vresult, "@considered", INT2NUM(result->considered));
   rb_iv_set (vresult, "@no_user_id", INT2NUM(result->no_user_id));
@@ -1504,6 +1597,9 @@ rb_s_gpgme_op_decrypt_result (VALUE dummy, VALUE vctx)
     rb_raise (rb_eArgError, "released ctx");
 
   result = gpgme_op_decrypt_result (ctx);
+  if (!result)
+    rb_raise (rb_eArgError, "no result");
+
   vresult = rb_class_new_instance (0, NULL, cDecryptResult);
   if (result->unsupported_algorithm)
     rb_iv_set (vresult, "@unsupported_algorithm",
@@ -1519,7 +1615,8 @@ rb_s_gpgme_op_decrypt_result (VALUE dummy, VALUE vctx)
       rb_iv_set (vrecipient, "@status", UINT2NUM(recipient->status));
       rb_ary_push (vrecipients, vrecipient);
     }
-  rb_iv_set (vresult, "@file_name", rb_str_new2 (result->file_name));
+  if (result->file_name)
+    rb_iv_set (vresult, "@file_name", rb_str_new2 (result->file_name));
   return vresult;
 }
 
@@ -1582,6 +1679,9 @@ rb_s_gpgme_op_verify_result (VALUE dummy, VALUE vctx)
     rb_raise (rb_eArgError, "released ctx");
 
   verify_result = gpgme_op_verify_result (ctx);
+  if (!verify_result)
+    rb_raise (rb_eArgError, "no result");
+
   vverify_result = rb_class_new_instance(0, NULL, cVerifyResult);
   rb_iv_set (vverify_result, "@signatures", vsignatures);
   for (signature = verify_result->signatures; signature;
@@ -1765,6 +1865,9 @@ rb_s_gpgme_op_sign_result (VALUE dummy, VALUE vctx)
     rb_raise (rb_eArgError, "released ctx");
 
   result = gpgme_op_sign_result (ctx);
+  if (!result)
+    rb_raise (rb_eArgError, "no result");
+
   vresult = rb_class_new_instance (0, NULL, cSignResult);
   vinvalid_signers = rb_ary_new ();
   rb_iv_set (vresult, "@invalid_signers", vinvalid_signers);
@@ -1880,6 +1983,9 @@ rb_s_gpgme_op_encrypt_result (VALUE dummy, VALUE vctx)
     rb_raise (rb_eArgError, "released ctx");
 
   result = gpgme_op_encrypt_result (ctx);
+  if (!result)
+    rb_raise (rb_eArgError, "no result");
+
   vresult = rb_class_new_instance (0, NULL, cEncryptResult);
   vinvalid_recipients = rb_ary_new ();
   rb_iv_set (vresult, "@invalid_recipients", vinvalid_recipients);
@@ -1988,6 +2094,96 @@ rb_s_gpgme_wait (VALUE dummy, VALUE vctx, VALUE rstatus, VALUE vhang)
   return Qnil;
 }
 
+#if defined(GPGME_VERSION_NUMBER) && GPGME_VERSION_NUMBER >= 0x010500
+static VALUE
+rb_s_gpgme_op_spawn_start (VALUE dummy, VALUE vctx, VALUE vfile,
+			   VALUE vargv, VALUE vdatain, VALUE vdataout,
+			   VALUE vdataerr, VALUE vflags)
+{
+  gpgme_ctx_t ctx;
+  const char *file;
+  const char **argv;
+  gpgme_data_t datain = NULL;
+  gpgme_data_t dataout = NULL;
+  gpgme_data_t dataerr = NULL;
+  gpgme_error_t err;
+
+  CHECK_KEYLIST_NOT_IN_PROGRESS(vctx);
+
+  UNWRAP_GPGME_CTX(vctx, ctx);
+  if (!ctx)
+    rb_raise (rb_eArgError, "released ctx");
+
+  file = StringValueCStr(vfile);
+
+  if (NIL_P(vargv))
+    argv = NULL;
+  else
+    {
+      int i;
+
+      argv = ALLOC_N(const char *, RARRAY_LEN(vargv) + 1);
+      for (i = 0; i < RARRAY_LEN(vargv); i++)
+	argv[i] = StringValueCStr(RARRAY_PTR(vargv)[i]);
+      argv[i] = NULL;
+    }
+
+  UNWRAP_GPGME_DATA(vdatain, datain);
+  UNWRAP_GPGME_DATA(vdataout, dataout);
+  UNWRAP_GPGME_DATA(vdataerr, dataerr);
+
+  err = gpgme_op_spawn_start (ctx, file, argv, datain, dataout, dataerr,
+			      NUM2INT(vflags));
+  if (argv)
+    xfree (argv);
+  return LONG2NUM(err);
+}
+
+static VALUE
+rb_s_gpgme_op_spawn (VALUE dummy, VALUE vctx, VALUE vfile,
+		     VALUE vargv, VALUE vdatain, VALUE vdataout,
+		     VALUE vdataerr, VALUE vflags)
+{
+  gpgme_ctx_t ctx;
+  const char *file;
+  const char **argv;
+  gpgme_data_t datain = NULL;
+  gpgme_data_t dataout = NULL;
+  gpgme_data_t dataerr = NULL;
+  gpgme_error_t err;
+
+  CHECK_KEYLIST_NOT_IN_PROGRESS(vctx);
+
+  UNWRAP_GPGME_CTX(vctx, ctx);
+  if (!ctx)
+    rb_raise (rb_eArgError, "released ctx");
+
+  file = StringValueCStr(vfile);
+
+  if (NIL_P(vargv))
+    argv = NULL;
+  else
+    {
+      int i;
+
+      argv = ALLOC_N(const char *, RARRAY_LEN(vargv) + 1);
+      for (i = 0; i < RARRAY_LEN(vargv); i++)
+	argv[i] = StringValueCStr(RARRAY_PTR(vargv)[i]);
+      argv[i] = NULL;
+    }
+
+  UNWRAP_GPGME_DATA(vdatain, datain);
+  UNWRAP_GPGME_DATA(vdataout, dataout);
+  UNWRAP_GPGME_DATA(vdataerr, dataerr);
+
+  err = gpgme_op_spawn (ctx, file, argv, datain, dataout, dataerr,
+			NUM2INT(vflags));
+  if (argv)
+    xfree (argv);
+  return LONG2NUM(err);
+}
+#endif
+
 void
 Init_gpgme_n (void)
 {
@@ -1995,6 +2191,10 @@ Init_gpgme_n (void)
 
   mGPGME = rb_define_module ("GPGME");
 
+#if defined(GPGME_VERSION_NUMBER) && GPGME_VERSION_NUMBER >= 0x010500
+  rb_define_module_function (mGPGME, "gpgme_get_dirinfo",
+			     rb_s_gpgme_get_dirinfo, 1);
+#endif
   rb_define_module_function (mGPGME, "gpgme_check_version",
 			     rb_s_gpgme_check_version, 1);
   rb_define_module_function (mGPGME, "gpgme_engine_check_version",
@@ -2123,6 +2323,16 @@ Init_gpgme_n (void)
 			     rb_s_gpgme_get_progress_cb, 3);
   rb_define_module_function (mGPGME, "gpgme_set_locale",
 			     rb_s_gpgme_set_locale, 3);
+#if defined(GPGME_VERSION_NUMBER) && GPGME_VERSION_NUMBER >= 0x010600
+  rb_define_module_function (mGPGME, "gpgme_set_offline",
+			     rb_s_gpgme_set_offline, 2);
+  rb_define_module_function (mGPGME, "gpgme_get_offline",
+			     rb_s_gpgme_get_offline, 1);
+  rb_define_module_function (mGPGME, "gpgme_set_status_cb",
+			     rb_s_gpgme_set_status_cb, 3);
+  rb_define_module_function (mGPGME, "gpgme_get_status_cb",
+			     rb_s_gpgme_get_status_cb, 3);
+#endif
 
   /* Key Management */
   rb_define_module_function (mGPGME, "gpgme_op_keylist_start",
@@ -2238,11 +2448,23 @@ Init_gpgme_n (void)
   rb_define_module_function (mGPGME, "gpgme_wait",
 			     rb_s_gpgme_wait, 3);
 
+  /* Running other Programs */
+#if defined(GPGME_VERSION_NUMBER) && GPGME_VERSION_NUMBER >= 0x010500
+  rb_define_module_function (mGPGME, "gpgme_op_spawn",
+			     rb_s_gpgme_op_spawn, 7);
+  rb_define_module_function (mGPGME, "gpgme_op_spawn_start",
+			     rb_s_gpgme_op_spawn_start, 7);
+#endif
+
   /* gpgme_pubkey_algo_t */
   rb_define_const (mGPGME, "GPGME_PK_RSA", INT2FIX(GPGME_PK_RSA));
   rb_define_const (mGPGME, "GPGME_PK_DSA", INT2FIX(GPGME_PK_DSA));
   rb_define_const (mGPGME, "GPGME_PK_ELG", INT2FIX(GPGME_PK_ELG));
   rb_define_const (mGPGME, "GPGME_PK_ELG_E", INT2FIX(GPGME_PK_ELG_E));
+  /* This algorithm was added in 1.5.0. */
+#ifdef GPGME_PK_ECC
+  rb_define_const (mGPGME, "GPGME_PK_ECC", INT2FIX(GPGME_PK_ECC));
+#endif
 
   /* gpgme_hash_algo_t */
   rb_define_const (mGPGME, "GPGME_MD_MD5", INT2FIX(GPGME_MD_MD5));
@@ -2251,6 +2473,10 @@ Init_gpgme_n (void)
   rb_define_const (mGPGME, "GPGME_MD_MD2", INT2FIX(GPGME_MD_MD2));
   rb_define_const (mGPGME, "GPGME_MD_TIGER", INT2FIX(GPGME_MD_TIGER));
   rb_define_const (mGPGME, "GPGME_MD_HAVAL", INT2FIX(GPGME_MD_HAVAL));
+  /* This algorithm was added in 1.5.0. */
+#ifdef GPGME_MD_SHA224
+  rb_define_const (mGPGME, "GPGME_MD_SHA224", INT2FIX(GPGME_MD_SHA224));
+#endif
   rb_define_const (mGPGME, "GPGME_MD_SHA256", INT2FIX(GPGME_MD_SHA256));
   rb_define_const (mGPGME, "GPGME_MD_SHA384", INT2FIX(GPGME_MD_SHA384));
   rb_define_const (mGPGME, "GPGME_MD_SHA512", INT2FIX(GPGME_MD_SHA512));
@@ -2497,6 +2723,11 @@ Init_gpgme_n (void)
   rb_define_const (mGPGME, "GPGME_PROTOCOL_ASSUAN",
 		   INT2FIX(GPGME_PROTOCOL_ASSUAN))
 #endif
+  /* This protocol was added in 1.5.0. */
+#ifdef GPGME_PROTOCOL_SPAWN
+  rb_define_const (mGPGME, "GPGME_PROTOCOL_SPAWN",
+                   INT2FIX(GPGME_PROTOCOL_SPAWN));
+#endif
 
   /* gpgme_status_code_t */
   rb_define_const (mGPGME, "GPGME_STATUS_EOF",
@@ -2653,6 +2884,21 @@ Init_gpgme_n (void)
   rb_define_const (mGPGME, "GPGME_STATUS_PKA_TRUST_GOOD",
 		   INT2FIX(GPGME_STATUS_PKA_TRUST_GOOD));
 #endif
+  /* These status codes were added in 1.5.0. */
+#if defined(GPGME_VERSION_NUMBER) && GPGME_VERSION_NUMBER >= 0x010500
+  rb_define_const (mGPGME, "GPGME_STATUS_PLAINTEXT_LENGTH",
+                   INT2FIX(GPGME_STATUS_PLAINTEXT_LENGTH));
+  rb_define_const (mGPGME, "GPGME_STATUS_MOUNTPOINT",
+                   INT2FIX(GPGME_STATUS_MOUNTPOINT));
+  rb_define_const (mGPGME, "GPGME_STATUS_PINENTRY_LAUNCHED",
+                   INT2FIX(GPGME_STATUS_PINENTRY_LAUNCHED));
+  rb_define_const (mGPGME, "GPGME_STATUS_ATTRIBUTE",
+                   INT2FIX(GPGME_STATUS_ATTRIBUTE));
+  rb_define_const (mGPGME, "GPGME_STATUS_BEGIN_SIGNING",
+                   INT2FIX(GPGME_STATUS_BEGIN_SIGNING));
+  rb_define_const (mGPGME, "GPGME_STATUS_KEY_NOT_CREATED",
+		   INT2FIX(GPGME_STATUS_KEY_NOT_CREATED));
+#endif
 
   /* The available keylist mode flags.  */
   rb_define_const (mGPGME, "GPGME_KEYLIST_MODE_LOCAL",
@@ -2673,6 +2919,11 @@ Init_gpgme_n (void)
   rb_define_const (mGPGME, "GPGME_KEYLIST_MODE_EPHEMERAL",
 		   INT2FIX(GPGME_KEYLIST_MODE_EPHEMERAL));
 #endif
+  /* This flag was added in 1.5.1. */
+#ifdef GPGME_KEYLIST_MODE_WITH_SECRET
+  rb_define_const (mGPGME, "GPGME_KEYLIST_MODE_WITH_SECRET",
+		   INT2FIX(GPGME_KEYLIST_MODE_WITH_SECRET));
+#endif
 
   /* The available flags for status field of gpgme_import_status_t.  */
   rb_define_const (mGPGME, "GPGME_IMPORT_NEW", INT2FIX(GPGME_IMPORT_NEW));
@@ -2705,4 +2956,34 @@ Init_gpgme_n (void)
   rb_define_const (mGPGME, "GPGME_PINENTRY_MODE_LOOPBACK",
                    INT2FIX(GPGME_PINENTRY_MODE_LOOPBACK));
 #endif
+
+  /* These flags were added in 1.5.0. */
+#if defined(GPGME_VERSION_NUMBER) && GPGME_VERSION_NUMBER >= 0x010500
+  rb_define_const (mGPGME, "GPGME_SPAWN_DETACHED",
+                   INT2FIX(GPGME_SPAWN_DETACHED));
+  rb_define_const (mGPGME, "GPGME_SPAWN_ALLOW_SET_FG",
+                   INT2FIX(GPGME_SPAWN_ALLOW_SET_FG));
+#endif
+
+  /* This flag was added in 1.2.0. */
+#ifdef GPGME_EXPORT_MODE_EXTERN
+  rb_define_const (mGPGME, "GPGME_EXPORT_MODE_EXTERN",
+                   INT2FIX(GPGME_EXPORT_MODE_EXTERN));
+#endif
+
+  /* This flag was added in 1.3.0. */
+#ifdef GPGME_EXPORT_MODE_MINIMAL
+  rb_define_const (mGPGME, "GPGME_EXPORT_MODE_MINIMAL",
+                   INT2FIX(GPGME_EXPORT_MODE_MINIMAL));
+#endif
+
+  /* These flags were added in 1.6.0. */
+#if defined(GPGME_VERSION_NUMBER) && GPGME_VERSION_NUMBER >= 0x010600
+  rb_define_const (mGPGME, "GPGME_EXPORT_MODE_SECRET",
+                   INT2FIX(GPGME_EXPORT_MODE_SECRET));
+  rb_define_const (mGPGME, "GPGME_EXPORT_MODE_RAW",
+                   INT2FIX(GPGME_EXPORT_MODE_RAW));
+  rb_define_const (mGPGME, "GPGME_EXPORT_MODE_PKCS12",
+                   INT2FIX(GPGME_EXPORT_MODE_PKCS12));
+#endif
 }
