@@ -8,16 +8,25 @@ module GPGME
     include KeyCommon
 
     def timestamp
-      Time.at(@timestamp)
+      case @timestamp
+      when -1, 0
+        # FIXME: add a special value for invalid timestamp, or throw an error
+        return nil
+      else
+        return Time.at(@timestamp)
+      end
+    end
+
+    def expires?
+      @expires != 0
     end
 
     def expires
-      Time.at(@expires)
+      expires? ? Time.at(@expires) : nil
     end
 
     def expired
-      return false if @expires == 0
-      @expires < Time.now.to_i
+      expires? && @expires < Time.now.to_i
     end
 
     def sha
