@@ -2,6 +2,9 @@
 require 'test_helper'
 
 describe GPGME::SubKey do
+  before do
+    skip unless GPGME::Engine.check_version GPGME::PROTOCOL_OpenPGP
+  end
 
   # We trust Key for common methods that come from KeyCommon
 
